{% load i18n %}<a href="{{ url }}" class="add-another" onclick="return showAddAnotherPopup(this);">
    <img src="{{ ADMIN_MEDIA_PREFIX }}img/admin/icon_addlink.gif" width="10" height="10" alt="{% trans 'Add Another' %}">
</a>